﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using BenchmarkDotNet.Attributes;
using BenchmarkDotNet.Running;

namespace EqualsImplementations {

	class X { }

	struct A {
		public int X;
		public int Y;
		public int Z;
	}

	struct B {
		public int X;
		public int Y;
		public X Z;
	}

	class C {
		public int X;
		public int Y;
		public int Z;
	}

	public class EqualsTests {
		A a1 = new A { X = 10, Y = 20, Z = 30 };
		A a2 = new A { X = 10, Y = 20, Z = 30 };
		A a3 = new A { X = 11, Y = 22, Z = 33 };

		static X x = new X();

		B b1 = new B { X = 10, Y = 20, Z = x };
		B b2 = new B { X = 10, Y = 20, Z = x };
		B b3 = new B { X = 11, Y = 22, Z = new X() };

		C c1 = new C { X = 10, Y = 20, Z = 30 };
		C c2 = new C { X = 10, Y = 20, Z = 30 };

		[Benchmark]
		public bool StructWithValueFieldsOnly_EqualInstances() {
			return a1.Equals(a2);
		}

		[Benchmark]
		public bool StructWithValueFieldsOnly_NonEqualInstances() {
			return a1.Equals(a3);
		}

		[Benchmark]
		public bool StructWithRefField_EqualInstances() {
			return b1.Equals(b2);
		}

		[Benchmark]
		public bool StructWithRefField_NonEqualInstances() {
			return b1.Equals(b3);
		}

		[Benchmark]
		public bool ClassWithDefaultEquals() {
			return c1.Equals(c2);
		}
	}

	class Program {
		static void Main(string[] args) {
			BenchmarkRunner.Run<EqualsTests>();

			var et = new EqualsTests();
			Console.WriteLine($"StructWithValueFieldsOnly_EqualInstances: {et.StructWithValueFieldsOnly_EqualInstances()}");
			Console.WriteLine($"StructWithValueFieldsOnly_NonEqualInstances: {et.StructWithValueFieldsOnly_NonEqualInstances()}");
			Console.WriteLine($"StructWithRefField_EqualInstances: {et.StructWithRefField_EqualInstances()}");
			Console.WriteLine($"StructWithValueFieldsOnly_NonEqualInstances: {et.StructWithValueFieldsOnly_NonEqualInstances()}");
			Console.WriteLine($"ClassWithDefaultEquals: {et.ClassWithDefaultEquals()}");
		}
	}
}
